unit Unit1;

{.$MODE Delphi}
{$mode objfpc}{$H+}

{.$define USE_OWN_CONNECTION} //use own database connection object.
                             //otherwise use ZEOS TZConnection
interface

uses
  LCLIntf, LCLType, LMessages, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ComCtrls, Spin, EditBtn, LazFileUtils, StdCtrls
  , ZConnection
  , ZlawConnection
  , ZlawMySQLBackup
  , ZSqlProcessor  //used from DoRestoreProcessor
  , fSQLBackupSettings
  ;

type

  { TForm1 }

  TForm1 = class(TForm)
    {$ifndef USE_OWN_CONNECTION}Connection:TZConnection;{$endif}
    btnBackup: TButton;
    btnDropCreate: TButton;
    btnRestoreOwn: TButton;
    lblCounter: TLabel;
    Memo1: TMemo;
    PageControl1: TPageControl;
    pnlBottom: TPanel;
    ProgressBar1: TProgressBar;
    tsBackup: TTabSheet;
    btnRestore: TButton;

    FBackupSet:TfrmSQLBackupSettings;


    procedure btnBackupClick(Sender: TObject);
    procedure btnDropCreateClick(Sender: TObject);
    procedure btnRestoreClick(Sender: TObject);
    procedure btnRestoreOwnClick(Sender: TObject);

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);

    //Event procedures from ZlawMySQLBackup to monitor the job
    procedure MyOnWriteHeader(Sender: TObject);
    procedure MyOnWriteFooter(Sender: TObject);
    procedure MyOnWriteTableData(const RecNo,RecCount:integer);
    procedure MyOnWriteTableDef(const ObjectName:string);

    procedure MyOnWriteTriggerDef(const ObjectName:string);
    procedure MyOnWriteProcedureDef(const ObjectName:string);
    procedure MyOnWriteViewDef(const ObjectName:string);

    procedure MyRestoreOnBeforeExecute(Processor: TZSQLProcessor; StatementIndex: Integer);
    procedure MyRestoreOnAfterExecute(Processor: TZSQLProcessor; StatementIndex: Integer);
    procedure MyRestoreOnError(Processor: TZSQLProcessor; StatementIndex: Integer; E: Exception;
    var ErrorHandleAction: TZErrorHandleAction);

  private
    FsExePath:string;
    sBackupFileName:string;
    //Restore with ZSqlProcessor
    procedure DoRestoreProcessor;

    //own restore procedure
    procedure DoRestoreOwn;

    //does the backup
    procedure DoBackup;

    //drops and creates an empty database for testing purposes
    //database name: testbackup
    procedure DropAndCreate;

  public
    { Public declarations }
    procedure Writedirecttofile(sFilename: string; s: string; bReset: boolean=false);
  end;

var
  Form1: TForm1;

implementation

{$R *.lfm}
uses ZScriptParser
     ,windows
     {$ifdef USE_OWN_CONNECTION}
     ,udmsqlconnectparent
     {$else}

     {$endif}
     ;

procedure TForm1.btnBackupClick(Sender: TObject);
begin

  memo1.clear;
  memo1.Append('************ Backup begins ***');

  DoBackup;

  memo1.Append('************ Backup ended ***');
end;

procedure TForm1.btnDropCreateClick(Sender: TObject);
begin
  DropAndCreate;
end;

procedure TForm1.btnRestoreClick(Sender: TObject);
begin
  DoRestoreProcessor;
end;

procedure TForm1.btnRestoreOwnClick(Sender: TObject);
begin
  DoRestoreOwn;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  FsExePath:= AppendPathDelim(ExtractFilePath(Application.ExeName));
  sBackupFileName:= FsExePath + 'backupdb.sql';
  memo1.clear;

  ProgressBar1.Visible:=false;
  lblCounter.Visible:=false;

  {$ifdef USE_OWN_CONNECTION}
  dmSQLConnectParent.autoconnect;
  {$else}
  Connection:=TZConnection.Create(self);

  {$endif}

  FBackupSet := TfrmSQLBackupSettings.Create(self);
  FBackupSet.DisableAlign;
  FBackupSet.Align := alClient;
  FBackupSet.ManualDock(tsBackup, nil, alClient);
  FBackupSet.EnableAlign;
  FBackupSet.FileNameEdit1.Text:=sBackupFileName;
  FBackupSet.DirectoryEdit1.Text:=AppendPathDelim(ExtractFilePath(Application.ExeName));
  FBackupSet.Show;

end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  {$ifndef USE_OWN_CONNECTION}
  freeandnil(Connection);
  {$endif}
  freeandnil(FBackupSet);
end;

procedure TForm1.MyOnWriteHeader(Sender: TObject);
begin
  Memo1.Append('Header Written');
end;

procedure TForm1.MyOnWriteFooter(Sender: TObject);
begin
  Memo1.Append('Footer Written');
end;

procedure TForm1.MyOnWriteTableData(const RecNo, RecCount: integer);
begin
  if RecNo = 1 then
  begin
     ProgressBar1.Min:=1;
     ProgressBar1.Max:=Reccount;
     ProgressBar1.Smooth:=true;
     ProgressBar1.Step:=1;
  end;

  if (RecNo mod 10) = 0 then           //smoothen Progressbar
    ProgressBar1.Position:=RecNo;

  lblCounter.Caption:= inttostr(RecNo)+'/'+inttostr(RecCount);
  Application.ProcessMessages;

  if  RecNo=RecCount then
  begin
     Memo1.Append('    written: '+inttostr(RecNo)+'/'+inttostr(RecCount));
  end;

  Application.ProcessMessages;
end;

procedure TForm1.MyOnWriteTableDef(const ObjectName: string);
begin
  Memo1.Append( 'Written Tabledef: '+ObjectName);
  lblCounter.Caption:='';
end;

procedure TForm1.MyOnWriteTriggerDef(const ObjectName: string);
begin
  Memo1.Append( 'Written Triggerdef '+ObjectName);
end;

procedure TForm1.MyOnWriteProcedureDef(const ObjectName: string);
begin
  Memo1.Append( 'Written ProcedureDef: '+ObjectName);
end;

procedure TForm1.MyOnWriteViewDef(const ObjectName: string);
begin
  Memo1.Append( 'Written ViewDef: '+ObjectName);
end;

procedure TForm1.MyRestoreOnBeforeExecute(Processor: TZSQLProcessor;
  StatementIndex: Integer);
begin
  Memo1.Append( ' Processing Statement #: '+inttostr(StatementIndex+1));
  lblCounter.Caption:= inttostr(StatementIndex+1)+'/'+inttostr(Processor.StatementCount);

  {$ifdef USE_OWN_CONNECTION}
    //writes the next statement to logfile before processing (debug purpose)
    WritedirecttoFile(FsExePath+'lastStatement.sql',Processor.Statements[StatementIndex],true);
  {$else}
     //use your own procedure to log
  {$endif}

  Progressbar1.Max:=Processor.StatementCount;
  Progressbar1.Position:=StatementIndex+1;
  Application.ProcessMessages;
end;

procedure TForm1.MyRestoreOnAfterExecute(Processor: TZSQLProcessor;
  StatementIndex: Integer);
begin
  Memo1.Append( '                      #: '+inttostr(StatementIndex+1)+' OK');
  lblCounter.Caption:= inttostr(StatementIndex+1)+'/'+inttostr( Processor.StatementCount);
  Application.ProcessMessages;
end;

procedure TForm1.MyRestoreOnError(Processor: TZSQLProcessor;
  StatementIndex: Integer; E: Exception;
  var ErrorHandleAction: TZErrorHandleAction);
begin
  { TODO : User action in case of an error is missing. React on error types
  TZErrorHandleAction = (eaFail, eaAbort, eaSkip, eaRetry);}
  Memo1.Append( 'Error at Statement #: '+inttostr(StatementIndex+1) +' '+E.Message);
  showMessage(E.Message);
end;

{Testing TZSQLProcessor.
seems to tokenize the SQL file at unwanted positions.
therfore i use DoRestoreOwn to tokenize and run the SQL statements for my own
as a quick hack without testing TSQLProcessor deeply.
}
procedure TForm1.DoRestoreProcessor;
{$ifdef USE_OWN_CONNECTION}
{$else}
var
  ZSQLProcessor1:TZSQLProcessor;
{$endif}
begin
  memo1.clear;

  {$ifdef USE_OWN_CONNECTION}
  try
    if MessageDlg('Are you sure? Your data could be lost!',mtInformation,[mbYes,mbNo],0)=mrYes then
    begin
      dmsqlconnectparent.ConnectDB(true,true);

      dmsqlconnectparent.ZSQLProcessor1.BeforeExecute:= @MyRestoreOnBeforeExecute;
      dmsqlconnectparent.ZSQLProcessor1.AfterExecute:=@MyRestoreOnAfterExecute;
      dmsqlconnectparent.ZSQLProcessor1.OnError:=@MyRestoreOnError;

      dmsqlconnectparent.ZSQLProcessor1.DelimiterType:=dtEmptyLine;
      dmsqlconnectparent.ZSQLProcessor1.Delimiter:=';';
      dmsqlconnectparent.ZSQLProcessor1.ParamCheck:=false;
      dmsqlconnectparent.ZSQLProcessor1.LoadFromFile(sBackupFileName);

      ProgressBar1.Position:=0;
      ProgressBar1.Visible:=true;
      lblCounter.Visible:=true;

      Memo1.Append('Restore starts');
      Memo1.Append('');
      Memo1.Append('Parsing Backup File -- please wait');

      dmsqlconnectparent.ZSQLProcessor1.Execute;

      Memo1.Append('Restore ended.');
      showmessage('Restore ended');
    end;
  finally
    dmsqlconnectparent.ZSQLProcessor1.BeforeExecute:= nil;
    dmsqlconnectparent.ZSQLProcessor1.AfterExecute:=nil;
    dmsqlconnectparent.ZSQLProcessor1.OnError:=nil;

    ProgressBar1.Visible:=false;
    lblCounter.Visible:=false;
  end;

  {$else}
   ZSQLProcessor1:=TZSQLProcessor.Create(self);
   try
     if MessageDlg('Are you sure? Your data could be lost!',mtInformation,[mbYes,mbNo],0)=mrYes then
     begin
       Connection.Connect;

       ZSQLProcessor1.BeforeExecute:= @MyRestoreOnBeforeExecute;
       ZSQLProcessor1.AfterExecute:=@MyRestoreOnAfterExecute;
       ZSQLProcessor1.OnError:=@MyRestoreOnError;

       ZSQLProcessor1.Connection:=Connection;
       ZSQLProcessor1.DelimiterType:=dtEmptyLine;
       ZSQLProcessor1.Delimiter:=';';
       ZSQLProcessor1.ParamCheck:=false;
       ZSQLProcessor1.LoadFromFile(sBackupFileName);

       ProgressBar1.Position:=0;
       ProgressBar1.Visible:=true;
       lblCounter.Visible:=true;

       Memo1.Append('Restore starts');
       Memo1.Append('');
       Memo1.Append('Parsing Backup File -- please wait');

       ZSQLProcessor1.Execute;

       Memo1.Append('Restore ended.');
       showmessage('Restore ended');
     end;
   finally
     freeandnil(ZSQLProcessor1);
     ProgressBar1.Visible:=false;
     lblCounter.Visible:=false;
   end;
  {$endif}
end;

procedure TForm1.DoRestoreOwn;
var
  s:string;
  sl:TStringlist;
  i:integer;
  bVerbose:boolean;
begin
  //Datei einlesen und an Strichpunkten teilen
  //Read file and cut/tokenize at semicolon
  sl:=TStringlist.Create;
  try
     sl.LoadFromFile(sBackupFileName);
     s:=sl.Text;
     sl.StrictDelimiter:=true;
     sl.Delimiter:=';';
     sl.DelimitedText:=s;

     //prepare Userinfo
     ProgressBar1.Visible:=true;
     lblCounter.Visible:=true;
     lblCounter.Caption:= '0/'+inttostr(sl.Count);
     ProgressBar1.Min:=0;
     ProgressBar1.Max:=sl.Count;
     ProgressBar1.Step:=1;
     Application.ProcessMessages;

     //alle statements abarbeiten und GUI updaten
     For i:=0 to sl.Count - 1 do
     begin
       lblCounter.Caption:= inttostr(i+1)+'/'+inttostr(sl.Count);
       ProgressBar1.StepIt;
       Application.ProcessMessages;

       if trim (sl[i])='' then continue;
       //if IsSqlComment( sl[i]) then continue;

       if bVerbose then  Memo1.Append(sl[i]);
       WritedirecttoFile(FsExePath+'lastStatement.sql', sl[i],true);

       {$ifdef USE_OWN_CONNECTION}

       if not dmSQLConnectparent.RunSQLExec(trim(sl[i]),{$I %CURRENTROUTINE%}+' at line '+{$I %LINE%},true) then
       begin
          WritedirecttoFile(FsExePath+'lastStatement.sql', 'ERROR !!!!',false);

          Memo1.Append('FEHLER: SQL-Anweisung fehlgeschlagen -- Sicherung muß wiederhergestellt werden');
          { TODO : User informieren}

          break;
       end
       else
       begin
         WritedirecttoFile(FsExePath+'lastStatement.sql', 'OK',false);
         if bVerbose then Memo1.Append('SQL Statement OK');

         { TODO : User informieren, loggen, anzeigen }
       end;
       {$else}
        if not Connection.ExecuteDirect(sl[i]) then
        begin
           WritedirecttoFile(FsExePath+'lastStatement.sql', 'ERROR !!!!',false);

           Memo1.Append('FEHLER: SQL-Anweisung fehlgeschlagen -- Sicherung muß wiederhergestellt werden');
           { TODO : User informieren}

           break;
        end
        else
        begin
          WritedirecttoFile(FsExePath+'lastStatement.sql', 'OK',false);
          if bVerbose then Memo1.Append('SQL Statement OK');

          { TODO : User informieren, loggen, anzeigen }
        end;
       {$endif}

     end;

  finally
    Memo1.Append(inttostr(sl.count)+' SQL Befehle wurden ausgeführt');
    freeandnil(sl);

    ProgressBar1.Visible:=false;
    lblCounter.Visible:=false;
    Application.ProcessMessages;
  end;
end;

procedure TForm1.DoBackup;
var
  Backup1: TZlawMySQLBackup;
begin

  Backup1:=TZlawMySQLBackup.Create(self);
  try

    with Backup1 do begin
       Backup1.pOptions:= FBackupSet.GetOptions(Backup1.pOptions);
       Backup1.pObjects:= FBackupSet.GetObjects(Backup1.pObjects);
       if lmbSetKeyChecksOn in Backup1.pOptions then showmessage('drin') else showmessage('nicht drin');

      {$ifdef USE_OWN_CONNECTION}
      Protocol := dmSQLConnectparent.prCredentials.protocol;
      HostName := dmSQLConnectparent.prCredentials.server ;
      Port := dmSQLConnectparent.prCredentials.port;
      Database := dmSQLConnectparent.prCredentials.DBname;
      User := dmSQLConnectparent.prCredentials.username;
      Password := dmSQLConnectparent.prCredentials.password;
      SQLHourGlass := True;
      LibraryLocation:= dmSQLConnectparent.prCredentials.LibraryLocation;

      {$else}
      Protocol := 'mysql-5';
      HostName := 'localhost' ;
      Port := 3306 ;
      Database := 'testbackup';
      User := 'root';
      Password := '';
      SQLHourGlass := True;
      LibraryLocation:= AppendPathDelim(ExtractFilePath(Application.ExeName)) + 'libmariadb.dll';
      {$endif}

      if not Connected then Connect;
    end;

    Backup1.OnWriteHeader:= @MyOnWriteHeader;
    Backup1.OnWriteFooter:= @MyOnWriteFooter;
    Backup1.OnWriteTableData:=@MyOnWriteTableData;
    Backup1.OnWriteTableDef:=@MyOnWriteTableDef;
    Backup1.OnWriteTriggerDef:=@MyOnWriteTriggerDef;
    Backup1.OnWriteProcedureDef:=@MyOnWriteProcedureDef;
    Backup1.OnWriteViewDef:=@MyOnWriteViewDef;

    ProgressBar1.Visible:=true;
    lblCounter.Visible:=true;

    sBackupFilename:=FBackupSet.FileNameEdit1.FileName;
    Backup1.Backup(sBackupFileName);

  finally
    freeandnil(Backup1);
    ProgressBar1.Visible:=false;
    lblCounter.Visible:=false;
  end;

end;

procedure TForm1.DropAndCreate;
var
  {$ifdef USE_OWN_CONNECTION}
    o:TdmSQLConnectParent;
  {$else}
  o:TZConnection;
  {$endif}

  s:string;
begin
  {$ifdef USE_OWN_CONNECTION}
  dmSQLConnectparent.DisConnectDB;

  o:=TdmSQLConnectParent.Create(self);
  try
    o.autoconnect;

    s:='DROP DATABASE IF EXISTS `testupdate`;';
    o.ZConnection1.ExecuteDirect(s);

    s:='CREATE DATABASE `testupdate` COLLATE '+QuotedStr('utf8_general_ci')+';';
    o.ZConnection1.ExecuteDirect(s);

  finally
    freeandnil(o);
  end;

  dmSQLConnectparent.autoconnect;

  {$else}
  Connection.Disconnect;

  o:=TZConnection.Create(self);
  try
    o.connect;

    s:='DROP DATABASE IF EXISTS `testupdate`;';
    o.ExecuteDirect(s);

    s:='CREATE DATABASE `testupdate` COLLATE '+QuotedStr('utf8_general_ci')+';';
    o.ExecuteDirect(s);

  finally
    freeandnil(o);
  end;

  Connection.Connect;
  {$endif}
end;

procedure TForm1.Writedirecttofile(sFilename: string; s: string; bReset: boolean
  );
var
   F: Textfile;
begin

    AssignFile(F,sFileName);
    {$I-}

    if not FileExists(sFileName) then
      Rewrite(F)
    else
      if (bReset ) then Rewrite(F);
      Append(F);

    {$i+}
    if IOResult = 0 then
    begin
      writeln(F,s);
      Flush(f);
      CloseFile(F);
    end
    else
      MessageDlg(sFileName + ' --File access error in procedure "Writedirecttofile" [KN]-- '+ SysErrorMessage(GetLastError) , mtWarning, [mbOk], 0);

end;

end.
