unit ZlawConnection;
{[CT]: i only introduced a possibility to switch off the encryption and left the rest untouched.
If using encryption install the package DCPcrypt
required Packages -> dcpcrypt, zcomponent, zparsesql}

{$MODE Delphi}
{.$define USE_ENCRYPTION}  //defines whether encryption is used at reading inifile

interface

uses
  SysUtils, Classes, ZConnection, ZDbcMySql;

type

  { TZlawConnection }

  TZlawConnection = class(TZConnection)
  private
    { Private declarations }
    FUserN:string;
    FIniName:String;
    FKey:string;
    FConnStr:string;
    FPW:string;
  protected
    { Protected declarations }
    function RetrieveConnStr(s:string):string;
  public
    { Public declarations }
    procedure Connect;override;
    procedure _StartTransaction;
    procedure _Commit;
    procedure _Rollback;
  published
    { Published declarations }
    property Key:string read FKey write FKey;
    property IniFileName:string read FIniName write FIniName;
  end;

procedure Register;

implementation

uses IniFiles
  {$ifdef USE_ENCRYPTION}
  , DCPcrypt
  , DCPrijndael
  , DCPmd5
  {$endif}
  ;

procedure Register;
begin
  RegisterComponents('Zeos Access', [TZlawConnection]);
end;

procedure TZlawConnection._StartTransaction;
begin
  FConnection.CreateStatement.Execute('START TRANSACTION');
end;

procedure TZlawConnection._Commit;
begin
  FConnection.CreateStatement.Execute('COMMIT');
end;

procedure TZlawConnection._Rollback;
begin
  FConnection.CreateStatement.Execute('ROLLBACK');
end;

function TZlawConnection.RetrieveConnStr(s: string): string;
{$ifdef USE_ENCRYPTION}
var ci:TDCP_rijndael;
{$endif}
begin
  {$ifdef USE_ENCRYPTION}
  ci:=TDCP_rijndael.Create(self);
  try
    ci.InitStr(FKey,TDCP_md5);
    result:=ci.DecryptString(s);
  finally
    ci.Burn;
    ci.Free;
  end;
  {$else}
  result:=s;
  exit;
  {$endif}

end;

procedure TZlawConnection.Connect;
var hini:TIniFile;
    x:string;
    i:Integer;
begin
  if (not Connected)
  then begin
    hini:=TIniFile.Create(FIniName);
//    showmessage(fininame);
    try
      Disconnect;
      if (FIniName<>'')
      then begin
        x:=hini.ReadString('DBLink','Catalog','');
        if x<>''
        then begin
          FConnStr:=x;
          Catalog:=RetrieveConnStr(FConnStr);
        end;
        x:=hini.ReadString('DBLink','Database','');
        if x<>''
        then Database:=x;
        x:=hini.ReadString('DBLink','HostName','');
        if x<>''
        then HostName:=x;
        x:=hini.ReadString('DBLink','User','');
        FUserN:=x;
        User:=x;
        FConnStr:=FUserN;
        User:=RetrieveConnStr(FConnStr);
        Password:=hini.ReadString('DBLink','Password','');
        x:=FPW;
        FPW:=Password;
        FConnStr:=Password;
        Password:=RetrieveConnStr(FConnStr);
        i:=hini.ReadInteger('DBLink','Port',0);
        if i<>0
        then Port:=i;
        x:=hini.ReadString('DBLink','Protocol','');
        if x<>''
        then Protocol:=x;
      end;
    finally
      hini.Free;
      inherited Connect;
    end;
  end;
end;

end.
