unit ZlawConnection;

interface

uses
  SysUtils, Classes, ZConnection, ZDBCMySQL;

type
  TZlawConnection = class(TZConnection)
  private
    { Private declarations }
    FUserN:string;
    FIniName:String;
    FKey:string;
    FConnStr:string;
    FPW:string;
  protected
    { Protected declarations }
    function RetrieveConnStr:string;
  public
    { Public declarations }
    procedure Connect;override;
    procedure _StartTransaction;
    procedure _Commit;
    procedure _Rollback;
  published
    { Published declarations }
    property Key:string read FKey write FKey;
    property IniFileName:string read FIniName write FIniName;
  end;

procedure Register;

implementation

uses IniFiles, DCPcrypt2, DCPrijndael, DCPmd5;

procedure Register;
begin
  RegisterComponents('Zeos Access', [TZlawConnection]);
end;

procedure TZlawConnection._StartTransaction;
begin
  FConnection.CreateStatement.Execute('START TRANSACTION');
end;

procedure TZlawConnection._Commit;
begin
  FConnection.CreateStatement.Execute('COMMIT');
end;

procedure TZlawConnection._Rollback;
begin
  FConnection.CreateStatement.Execute('ROLLBACK');
end;

function TZlawConnection.RetrieveConnStr:string;
var ci:TDCP_rijndael;
begin
  ci:=TDCP_rijndael.Create(self);
  try
    ci.InitStr(FKey,TDCP_md5);
    result:=ci.DecryptString(FConnStr);
  finally
    ci.Burn;
    ci.Free;
  end;
end;

procedure TZlawConnection.Connect;
var hini:TIniFile;
    x:string;
    i:Integer;
begin
  if (not Connected)
  then begin
    hini:=TIniFile.Create(FIniName);
//    showmessage(fininame);
    try
      Disconnect;
      if (FIniName<>'')
      then begin
        x:=hini.ReadString('DBLink','Catalog','');
        if x<>''
        then begin
          FConnStr:=x;
          Catalog:=RetrieveConnStr;
        end;
        x:=hini.ReadString('DBLink','Database','');
        if x<>''
        then Database:=x;
        x:=hini.ReadString('DBLink','HostName','');
        if x<>''
        then HostName:=x;
        x:=hini.ReadString('DBLink','User','');
        FUserN:=x;
        User:=x;
        FConnStr:=FUserN;
        User:=RetrieveConnStr;
        Password:=hini.ReadString('DBLink','Password','');
        x:=FPW;
        FPW:=Password;
        FConnStr:=Password;
        Password:=RetrieveConnStr;
        i:=hini.ReadInteger('DBLink','Port',0);
        if i<>0
        then Port:=i;
        x:=hini.ReadString('DBLink','Protocol','');
        if x<>''
        then Protocol:=x;
      end;
    finally
      hini.Free;
      inherited Connect;
    end;
  end;
end;

end.
