unit ZlawMySQLBackup;

{.$MODE Delphi}
{$mode objfpc}{$H+}

{.$define USE_TZCONNECTION} //uses TZConnection as Ancestor -- otherwise TZlawConnection is used

interface

uses
  SysUtils, Classes, ZConnection, ZDataset
  {$ifdef USE_TZCONNECTION}
  {$else}
  , ZlawConnection
  {$endif}
  ;

type
TZlawMySQLBackupOption = (lmbDropDatabase, lmbCreateDatabase, lmbUseDatabase,
    lmbFlushLogsBeforeDump, lmbSetKeyChecksOn, lmbDropObjects,
    lmbLockAroundInserts, lmbDumpData, lmbDisableKeysAtInsert, lmbRestoreToNewDatabase,
    lmbInsertWithFields, lmbReplaceBadCharacters, lmbOverwriteBackupfile);
  TZlawMySQLBackupOptions = set of TZlawMySQLBackupOption;

  TZlawMySQLBackupObject = (lmboTables, lmboViews, lmboProcedures,
    lmboFunctions, lmboTriggers);
  TZlawMySQLBackupObjects = set of TZlawMySQLBackupObject;

  TZlawMySQLBackupNotifyEvent = procedure(const ObjectName: string) of object;
  TZlawMySQLBackupDumpDataEvent = procedure(const RecNo, RecCount: integer) of object;


  { TZlawMySQLBackup    }

  {$ifdef USE_TZCONNECTION}
  TZlawMySQLBackup = class(TZConnection)
  {$else}
  TZlawMySQLBackup = class(TZlawConnection)
  {$endif}

  private
    FsBaseDelimiter: string;   //the base Delimiter   e.g. ';'  ..semicolon
    FsDelimiter: string;       // Delimiter for SQL Statements (FsBaseDelimiter+crlf+crlf)
    FQuery, FTables: TZReadOnlyQuery;
    FsNewDbName: string;
    FFileStream: TFileStream;
    FMemStream: TMemoryStream;
    FTableName: string;
    FBuffer: string;
    FObjects: TZlawMySQLBackupObjects;
    FOptions: TZlawMySQLBackupOptions;
    FOnWriteFooter: TNotifyEvent;
    FOnWriteHeader: TNotifyEvent;
    FOnWriteTableData: TZlawMySQLBackupDumpDataEvent;
    FOnWriteTableDef: TZlawMySQLBackupNotifyEvent;
    FOnWriteTriggerDef: TZlawMySQLBackupNotifyEvent;
    FOnWriteProcedureDef: TZlawMySQLBackupNotifyEvent;
    FOnWriteViewDef: TZlawMySQLBackupNotifyEvent;
    procedure SetBaseDelimiter(AValue: string);
    procedure SetNewDbName(AValue: string);
    procedure SetObjects(const Value: TZlawMySQLBackupObjects);
    procedure SetOptions(const Value: TZlawMySQLBackupOptions);
    procedure SetOnWriteFooter(const Value: TNotifyEvent);
    procedure SetOnWriteHeader(const Value: TNotifyEvent);
    procedure SetOnWriteProcedureDef(const Value: TZlawMySQLBackupNotifyEvent);
    procedure SetOnWriteTableData(const Value: TZlawMySQLBackupDumpDataEvent);
    procedure SetOnWriteTableDef(const Value: TZlawMySQLBackupNotifyEvent);
    procedure SetOnWriteTriggerDef(const Value: TZlawMySQLBackupNotifyEvent);
    procedure SetOnWriteViewDef(const Value: TZlawMySQLBackupNotifyEvent);
    procedure SetDelimiter;
  protected
    procedure WriteHeader;
    procedure WriteFooter;
    procedure WriteInsertHeader(const tablename: string);
    procedure WriteInsertValues;
    procedure WriteTableDefs;
    procedure WriteTableData(const tablename: string);
    procedure WriteViewDefs;
    procedure WriteProcedureDefs;
    procedure WriteTriggerDefs;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Backup(const filename: string);
    property TableName: string read FTableName write FTableName;
  published
    property psDelimiter: string read FsBaseDelimiter write SetBaseDelimiter;
    property psNewDbName: string read FsNewDbName write SetNewDbName;
    property pOptions: TZlawMySQLBackupOptions read FOptions write SetOptions;
    property pObjects: TZlawMySQLBackupObjects read FObjects write SetObjects;
    property OnWriteHeader: TNotifyEvent read FOnWriteHeader write SetOnWriteHeader;
    property OnWriteTableDef: TZlawMySQLBackupNotifyEvent  read FOnWriteTableDef write SetOnWriteTableDef;
    property OnWriteTableData: TZlawMySQLBackupDumpDataEvent read FOnWriteTableData write SetOnWriteTableData;
    property OnWriteViewDef: TZlawMySQLBackupNotifyEvent read FOnWriteViewDef write SetOnWriteViewDef;
    property OnWriteProcedureDef: TZlawMySQLBackupNotifyEvent read FOnWriteProcedureDef write SetOnWriteProcedureDef;
    property OnWriteTriggerDef: TZlawMySQLBackupNotifyEvent read FOnWriteTriggerDef write SetOnWriteTriggerDef;
    property OnWriteFooter: TNotifyEvent read FOnWriteFooter write SetOnWriteFooter;
  end;

procedure Register;

function Backtick(s: string): string;

implementation

uses DB, DateUtils, Dialogs, controls
  ;

const
  crlf = #13#10;

procedure Register;
begin
  RegisterComponents('Zeos Access', [TZlawMySQLBackup]);
end;

function Backtick(s: string): string;
begin
  Result := '`' + s + '`';
  //result:= #96 + s + #96;
end;

{ TZlawMySQLBackup }
constructor TZlawMySQLBackup.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FObjects := [lmboTables, lmboViews, lmboProcedures, lmboFunctions, lmboTriggers];
  FOptions := [lmbDropDatabase, lmbCreateDatabase, lmbUseDatabase, lmbFlushLogsBeforeDump,
    lmbSetKeyChecksOn, lmbDropObjects, lmbLockAroundInserts, lmbDumpData,
    lmbDisableKeysAtInsert, lmbReplaceBadCharacters];

  FTables := TZReadOnlyQuery.Create(self);
  FTables.Connection := self;
  FQuery := TZReadOnlyQuery.Create(self);
  FQuery.Connection := self;
  FsBaseDelimiter :=';';
  SetDelimiter;
  FsNewDbName:='no_new databasename_defined';
end;

destructor TZlawMySQLBackup.Destroy;
begin
  FTables.Free;
  FQuery.Free;
  inherited Destroy;
end;

procedure TZlawMySQLBackup.WriteHeader;
var
  sDbName:string;
begin
{
-- --------------------------------------------------------
-- Host:                         127.0.0.1
-- Server Version:               10.1.13-MariaDB - mariadb.org binary distribution
-- Server Betriebssystem:        Win32
-- HeidiSQL Version:             9.3.0.4984
-- --------------------------------------------------------

/*!40101 SET @OLD_CHARACTER_SET_CLIENT=@@CHARACTER_SET_CLIENT */;
/*!40101 SET NAMES utf8mb4 */;
/*!40014 SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0 */;
/*!40101 SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE='NO_AUTO_VALUE_ON_ZERO' */;
}


  if lmbRestoreToNewDatabase in FOptions then
  begin
     sDbName:=FsNewDbName; //use another Databasename for export
  end
  else
    sDbName:=Database;  //use database name from ZConnection


  FBuffer := '';
  if lmbSetKeyChecksOn in FOptions then
  begin
    FBuffer := FBuffer + 'SET @OLD_UNIQUE_CHECKS=@@UNIQUE_CHECKS, UNIQUE_CHECKS=0'+ FsDelimiter;
    FBuffer := FBuffer + 'SET @OLD_FOREIGN_KEY_CHECKS=@@FOREIGN_KEY_CHECKS, FOREIGN_KEY_CHECKS=0' + FsDelimiter;
  end;

  FBuffer := FBuffer + 'SET @OLD_SQL_MODE=@@SQL_MODE, SQL_MODE=''NO_AUTO_VALUE_ON_ZERO''' + FsDelimiter;

  if lmbDropDatabase in FOptions then
    FBuffer := FBuffer + 'DROP DATABASE IF EXISTS ' + Backtick(sDbName) + FsDelimiter;

  if lmbCreateDatabase in FOptions then
    FBuffer := FBuffer + 'CREATE DATABASE IF NOT EXISTS ' + Backtick(sDbName) + FsDelimiter;

  if lmbUseDatabase in FOptions then
    FBuffer := FBuffer + 'USE ' + Backtick(sDbName) + FsDelimiter;

  FFileStream.Write(FBuffer[1], Length(FBuffer));
  if Assigned(FOnWriteHeader) then
    FOnWriteHeader(self);
end;

procedure TZlawMySQLBackup.WriteFooter;
begin
  FBuffer := 'SET SQL_MODE=@OLD_SQL_MODE' + FsDelimiter;

  if lmbSetKeyChecksOn in FOptions then
  begin
    FBuffer := FBuffer + 'SET FOREIGN_KEY_CHECKS=@OLD_FOREIGN_KEY_CHECKS' + FsDelimiter;
    FBuffer := FBuffer + 'SET UNIQUE_CHECKS=@OLD_UNIQUE_CHECKS' + FsDelimiter;
  end;

  FFileStream.Write(FBuffer[1], Length(FBuffer));
  if Assigned(FOnWriteFooter) then
    FOnWriteFooter(self);
end;

procedure TZlawMySQLBackup.WriteInsertHeader(const tablename: string);
var
  i,iFieldcount:integer;
begin
  //iFieldcount:=FQuery.Fields[0].AsInteger;
  iFieldcount:=FQuery.FieldCount;

  FBuffer := 'INSERT INTO ' + Backtick(tablename);

  if lmbInsertWithFields in FOptions then
  begin
     FBuffer := FBuffer + ' ( ';
     for i:=0 to iFieldcount-1 do
     begin
       FBuffer := FBuffer + Backtick(FQuery.Fields[i].FieldName);
       if i < iFieldcount-1 then FBuffer := FBuffer +', ';
     end;
     FBuffer := FBuffer + ' ) ';
  end;

  FBuffer := FBuffer + ' VALUES ';

  FBuffer := FBuffer + crlf; //new line for data values
  FFileStream.Write(FBuffer[1], Length(FBuffer));
end;

procedure TZlawMySQLBackup.WriteInsertValues;
var
  cikl: integer;
  comma: string;
  //sindent: string;
  function IntToStrZero(mit, hossz: longint): string;
  var
    work: string;
    cikl: integer;
  begin
    Result := '';
    Str(mit, work);
    if Length(work) < hossz then
    begin
      for cikl := 1 to hossz - Length(work) do
        Result := Result + '0';
      Result := Result + work;
    end
    else
      Result := work;
  end;

  function UnEscapeString(const szoveg, escape: string): string;
  begin
    //result:=RS(szoveg,'\',escape+escape);
    //result:=RS(result,'''',escape+'''');
    //result:=RS(result,'"',escape+'"');
    Result := StringReplace(szoveg, '\', escape + escape, [rfReplaceAll, rfIgnoreCase]);
    Result := StringReplace(Result, '''', escape + '''', [rfReplaceAll, rfIgnoreCase]);
    Result := StringReplace(Result, '"', escape + '"', [rfReplaceAll, rfIgnoreCase]);
  end;

  function ReplBadChar(const s: string): string;
  begin
    Result := StringReplace(s, FsBaseDelimiter, '_', [rfReplaceAll, rfIgnoreCase]);
  end;

  function HexValue(BinValue: String): String;
  var
    BinLen: Integer;
    Ansi: AnsiString;
  begin
    // Return a binary value as hex AnsiString
    Ansi := AnsiString(BinValue);
    BinLen := Length(Ansi);
    if BinLen = 0 then begin
      //Result := Connection.EscapeString('');
      Result := QuotedStr('');
    end else begin
      SetLength(Result, BinLen*2);
      BinToHex(PAnsiChar(Ansi), PChar(Result), BinLen);
      Result := '0x' + Result;
    end;
  end;

begin
  for cikl := 0 to FQuery.FieldCount - 1 do
  begin
    if cikl = FQuery.FieldCount - 1 then
      comma := ''
    else
      comma := ',';

    if FQuery.Fields[cikl].IsNull then
    begin
      FBuffer := 'NULL' + comma;
      FFileStream.Write(FBuffer[1], Length(FBuffer));
    end
    else
      case FQuery.Fields[cikl].DataType of
        ftString:
        begin
          FBuffer := FQuery.Fields[cikl].AsString;
          if lmbReplaceBadCharacters in FOptions then
            FBuffer := ReplBadChar(FBuffer);

          FBuffer := '''' + UnEscapeString(FBuffer, '\') + '''' + comma;
          FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
        ftSmallInt, ftInteger, ftWord, ftLargeInt:
        begin
          FBuffer := IntToStrZero(FQuery.Fields[cikl].AsInteger, 1) + comma;
          FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
        ftFloat, ftCurrency:
        begin
          FBuffer := StringReplace(
            FormatFloat('0', FQuery.Fields[cikl].AsFloat), ',', '.', [rfReplaceAll]) + comma;
          FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
        ftDate:
        begin
          FBuffer := '''' + IntToStr(YearOf(FQuery.Fields[cikl].AsDateTime)) +
            '-' + IntToStrZero(MonthOf(FQuery.Fields[cikl].AsDateTime), 2) + '-' +
            IntToStrZero(DayOf(FQuery.Fields[cikl].AsDateTime), 2) + '''' + comma;
          FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
        ftDateTime:
        begin
          FBuffer := '''' + IntToStr(YearOf(FQuery.Fields[cikl].AsDateTime)) +
            '-' + IntToStrZero(MonthOf(FQuery.Fields[cikl].AsDateTime), 2) + '-' +
            IntToStrZero(DayOf(FQuery.Fields[cikl].AsDateTime), 2) +
            ' ' + IntToStrZero(HourOf(FQuery.Fields[cikl].AsDateTime), 2) + ':' +
            IntToStrZero(MinuteOf(FQuery.Fields[cikl].AsDateTime), 2) +
            ':' + IntToStrZero(SecondOf(FQuery.Fields[cikl].AsDateTime), 2) + '''' + comma;
          FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
        ftBlob:
        begin
          { TODO : Export bei Heidi wenn leer:  _binary ''
                   sonst _binary 0xC12she41563

                   dtcBinary, dtcSpatial: begin
                                     BinContent := Data.HexValue(i);
                                     if Length(BinContent) > 0 then
                                       Row := Row + '_binary ' + BinContent
                                     else
                                       Row := Row + esc('');
                                   end;


                                   function TDBQuery.HexValue(BinValue: String): String;
                                   var
                                     BinLen: Integer;
                                     Ansi: AnsiString;
                                   begin
                                     // Return a binary value as hex AnsiString
                                     Ansi := AnsiString(BinValue);
                                     BinLen := Length(Ansi);
                                     if BinLen = 0 then begin
                                       Result := Connection.EscapeString('');
                                     end else begin
                                       SetLength(Result, BinLen*2);
                                       BinToHex(PAnsiChar(Ansi), PChar(Result), BinLen);
                                       Result := '0x' + Result;
                                     end;
                                   end;

                   }
          //[KN] FBuffer := '0x';

          FBuffer := HexValue(FQuery.Fields[cikl].AsString);
          if Length(FBuffer) > 0 then
            FBuffer:= '_binary ' + FBuffer
           else
            FBuffer:= QuotedStr('');

          FFileStream.Write(FBuffer[1], Length(FBuffer));

          //FBuffer := '_binary 0x';
          //FFileStream.Write(FBuffer[1], Length(FBuffer));
          //
          //FMemStream.Clear;
          //TBlobField(FQuery.Fields[cikl]).SaveToStream(FMemStream);
          //ConvertStreamToHex(FMemStream, FFileStream);
          //FBuffer := comma;
          //FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
        ftMemo:
        begin

          FBuffer := '''' + UnEscapeString(FQuery.Fields[cikl].AsString, '\') +
            '''' + comma;

          if lmbReplaceBadCharacters in FOptions then
            FBuffer := ReplBadChar(FBuffer);

          FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
        else
        begin
          FBuffer := IntToStrZero(Ord(FQuery.Fields[cikl].DataType), 2) + comma;
          FFileStream.Write(FBuffer[1], Length(FBuffer));
        end;
      end;
  end;

end;

procedure TZlawMySQLBackup.WriteTableDefs;
begin
  if lmboTables in FObjects then
  begin
    FTables.SQL.Text := 'SHOW FULL TABLES WHERE Table_type=''BASE TABLE''';
    FTables.Open;
    while not FTables.EOF do
    begin
      if (FTables.Fields[0].AsString = TableName) or (TableName = '') then
      begin
        FBuffer := '';
        if lmbDropObjects in FOptions then
          FBuffer := FBuffer + 'DROP TABLE IF EXISTS ' + Backtick(FTables.Fields[0].AsString) + FsDelimiter;
        FQuery.SQL.Text := 'SHOW CREATE TABLE ' + Backtick(FTables.Fields[0].AsString);
        FQuery.Open;
        FBuffer := FBuffer + FQuery.Fields[1].AsString + FsDelimiter;
        FFileStream.Write(FBuffer[1], Length(FBuffer));

        if Assigned(FOnWriteTableDef) then
          FOnWriteTableDef(FTables.Fields[0].AsString);

        { TODO : schreiben der Tabellendaten in eigene Files }
        WriteTableData(FTables.Fields[0].AsString);
      end;
      FTables.Next;
    end;
  end;
end;

procedure TZlawMySQLBackup.WriteTableData(const tablename: string);

var
  //comma: string;
  sindent: string;
  //cikl: integer;
  reccount: integer;

begin
  sindent := '  ';
  if lmbDumpData in FOptions then
  begin
    FBuffer := '';

    //Count records first
    FQuery.SQL.Text := 'SELECT COUNT(*) FROM ' + Backtick(tablename);
    FQuery.Open;
    reccount := FQuery.Fields[0].AsInteger;

    if reccount > 0 then    //don't write anything if reccount = 0
    begin

      if lmbLockAroundInserts in FOptions then
        FBuffer := FBuffer + 'LOCK TABLES ' + Backtick(tablename) + ' WRITE' + FsDelimiter;

      if lmbDisableKeysAtInsert in FOptions then
        FBuffer := FBuffer + 'ALTER TABLE ' + Backtick(tablename) + ' DISABLE KEYS' + FsDelimiter;

      FFileStream.Write(FBuffer[1], Length(FBuffer));

      FQuery.SQL.Text := 'SELECT * FROM ' + Backtick(tablename);
      FQuery.Open;
      WriteInsertHeader(tablename);  //write INSERT Header

      while not FQuery.EOF do
      begin
        FBuffer := sIndent + '(';
        FFileStream.Write(FBuffer[1], Length(FBuffer));

        WriteInsertValues;  //Write values without brackets

        if Assigned(FOnWriteTableData) then
          FOnWriteTableData(FQuery.RecNo, reccount);

        FQuery.Next;
        if FQuery.EOF then
          FBuffer := ')' + FsDelimiter
        else
          FBuffer := '),' + crlf;

        FFileStream.Write(FBuffer[1], Length(FBuffer));

        if lmbDisableKeysAtInsert in FOptions then
          FBuffer := 'ALTER TABLE ' + Backtick(tablename) + ' ENABLE KEYS' + FsDelimiter;

        if lmbLockAroundInserts in FOptions then
          FBuffer := FBuffer + 'UNLOCK TABLES' + FsDelimiter;
      end;
    end; //reccount > 0

    FFileStream.Write(FBuffer[1], Length(FBuffer));
  end;
end;

procedure TZlawMySQLBackup.WriteViewDefs;
begin
  if lmboViews in FObjects then
  begin
    FTables.SQL.Text := 'SHOW FULL TABLES WHERE Table_type=''VIEW''';
    FTables.Open;
    while not FTables.EOF do
    begin
      FBuffer := '';
      if lmbDropObjects in FOptions then
        FBuffer := FBuffer + 'DROP VIEW IF EXISTS ' + Backtick(FTables.Fields[0].AsString) + FsDelimiter;

      FQuery.SQL.Text := 'SHOW CREATE VIEW ' + Backtick(FTables.Fields[0].AsString);
      FQuery.Open;
      FBuffer := FBuffer + FQuery.Fields[1].AsString + FsDelimiter;
      FFileStream.Write(FBuffer[1], Length(FBuffer));
      if Assigned(FOnWriteViewDef) then
        FOnWriteViewDef(FTables.Fields[0].AsString);
      FTables.Next;
    end;
  end;
end;

procedure TZlawMySQLBackup.WriteProcedureDefs;
var
  db: string;
  proclist, funclist: TStringList;
  cikl: integer;
begin
  if (lmboProcedures in FObjects) or (lmboFunctions in FObjects) then
  begin
    proclist := TStringList.Create;
    funclist := TStringList.Create;
    try
      db := Database;
      Disconnect;
      Database := 'information_schema';
      Connect;
      FTables.SQL.Text := 'SELECT * FROM ROUTINES WHERE ROUTINE_SCHEMA=' + QuotedStr(db);
      FTables.Open;
      while not FTables.EOF do
      begin
        if FTables.FieldByName('ROUTINE_TYPE').AsString = 'PROCEDURE' then
          proclist.Add(FTables.FieldByName('ROUTINE_NAME').AsString)
        else
        if FTables.FieldByName('ROUTINE_TYPE').AsString = 'FUNCTION' then
          funclist.Add(FTables.FieldByName('ROUTINE_NAME').AsString);

        FTables.Next;
      end;
      Disconnect;
      Database := db;
      Connect;
      if lmboProcedures in FObjects then
      begin
        for cikl := 0 to proclist.Count - 1 do
        begin
          FTables.SQL.Text := 'SHOW CREATE PROCEDURE ' + Backtick(proclist[cikl]);
          FTables.Open;
          FBuffer := '';
          if lmbDropObjects in FOptions then
            FBuffer := FBuffer + 'DROP PROCEDURE IF EXISTS ' + Backtick(proclist[cikl]) + FsDelimiter;
          FBuffer := FBuffer + FTables.Fields[2].AsString + FsDelimiter;
          FFileStream.Write(FBuffer[1], Length(FBuffer));
          if Assigned(FOnWriteProcedureDef) then
            FOnWriteProcedureDef(proclist[cikl]);
        end;
      end;
      if lmboFunctions in FObjects then
      begin
        for cikl := 0 to funclist.Count - 1 do
        begin
          FTables.SQL.Text := 'SHOW CREATE FUNCTION ' + Backtick(funclist[cikl]);
          FTables.Open;
          FBuffer := '';

          if lmbDropObjects in FOptions then
            FBuffer := Fbuffer + 'DROP FUNCTION IF EXISTS ' + Backtick(funclist[cikl]) + FsDelimiter;

          FBuffer := FBuffer + FTables.Fields[2].AsString + FsDelimiter;
          FFileStream.Write(FBuffer[1], Length(FBuffer));

          if Assigned(FOnWriteProcedureDef) then
            FOnWriteProcedureDef(proclist[cikl]);
        end;
      end;
    finally
      funclist.Free;
      proclist.Free;
    end;
  end;
end;

procedure TZlawMySQLBackup.WriteTriggerDefs;
begin
  if lmboTriggers in FObjects then
  begin
    FTables.SQL.Text := 'SHOW TRIGGERS';
    FTables.Open;
    while not FTables.EOF do
    begin
      FBuffer := '';
      if lmbDropObjects in FOptions then
        FBuffer := FBuffer + 'DROP TRIGGER IF EXISTS ' + Backtick(FTables.FieldByName('Trigger').AsString) + FsDelimiter;

      FBuffer := FBuffer
        + 'CREATE TRIGGER ' + Backtick(FTables.FieldByName('Trigger').AsString)
        +    ' ' +  Backtick(FTables.FieldByName('Timing').AsString)
        +    ' ' +  Backtick(FTables.FieldByName('Event').AsString)
        + ' ON ' + Backtick(FTables.FieldByName('Table').AsString)
        + ' FOR EACH ROW ' + Backtick(FTables.FieldByName('Statement').AsString) + FsDelimiter;

      FFileStream.Write(FBuffer[1], Length(FBuffer));

      if Assigned(FOnWriteTriggerDef) then
        FOnWriteTriggerDef(FTables.FieldByName('Trigger').AsString);

      FTables.Next;
    end;
  end;
end;

procedure TZlawMySQLBackup.Backup(const filename: string);
begin
  if not (lmbOverwriteBackupfile in FOptions) then
    if FileExists(filename) then
        if MessageDlg('Frage', 'Backup Datei existiert -- überschreiben?', mtConfirmation,
           [mbYes, mbNo],0) = mrNo
        then exit;

  FFileStream := TFileStream.Create(filename, fmCreate);
  FMemStream := TMemoryStream.Create;
  try
    if lmbFlushLogsBeforeDump in FOptions then
    begin
      FQuery.SQL.Text := 'FLUSH LOGS';
      FQuery.ExecSQL;
    end;
    WriteHeader;
    WriteTableDefs; //including data dump
    WriteViewDefs;
    WriteProcedureDefs;
    WriteTriggerDefs;
    WriteFooter;
  finally
    FMemStream.Free;
    FFileStream.Free;
  end;
end;

procedure TZlawMySQLBackup.SetObjects(const Value: TZlawMySQLBackupObjects);
begin
  FObjects := Value;
end;

procedure TZlawMySQLBackup.SetBaseDelimiter(AValue: string);
begin
  if FsBaseDelimiter = AValue then
    Exit;
  FsBaseDelimiter := AValue;
  SetDelimiter;
end;

procedure TZlawMySQLBackup.SetNewDbName(AValue: string);
begin
  if FsNewDbName=AValue then Exit;
  FsNewDbName:=AValue;
end;

procedure TZlawMySQLBackup.SetOptions(const Value: TZlawMySQLBackupOptions);
begin
  FOptions := Value;
end;

procedure TZlawMySQLBackup.SetOnWriteFooter(const Value: TNotifyEvent);
begin
  FOnWriteFooter := Value;
end;

procedure TZlawMySQLBackup.SetOnWriteHeader(const Value: TNotifyEvent);
begin
  FOnWriteHeader := Value;
end;

procedure TZlawMySQLBackup.SetOnWriteProcedureDef(
  const Value: TZlawMySQLBackupNotifyEvent);
begin
  FOnWriteProcedureDef := Value;
end;

procedure TZlawMySQLBackup.SetOnWriteTableData(
  const Value: TZlawMySQLBackupDumpDataEvent);
begin
  FOnWriteTableData := Value;
end;

procedure TZlawMySQLBackup.SetOnWriteTableDef(
  const Value: TZlawMySQLBackupNotifyEvent);
begin
  FOnWriteTableDef := Value;
end;

procedure TZlawMySQLBackup.SetOnWriteTriggerDef(
  const Value: TZlawMySQLBackupNotifyEvent);
begin
  FOnWriteTriggerDef := Value;
end;

procedure TZlawMySQLBackup.SetOnWriteViewDef(const Value: TZlawMySQLBackupNotifyEvent);
begin
  FOnWriteViewDef := Value;
end;

procedure TZlawMySQLBackup.SetDelimiter;
begin
  FsDelimiter:=FsBaseDelimiter+crlf+crlf;
end;

end.
