unit lawStream;

interface

uses Classes, Db, ExtCtrls, Graphics;

procedure ReadDataFromField(Field:TBlobField;DataStream:TMemoryStream;DeCompress:boolean);
procedure WriteDataToField(Field:TBlobField;DataStream:TMemoryStream;Compress:boolean);
procedure ReadImageFromField(Field:TBlobField;Img:TImage;ImageType:string;Compress:boolean);
procedure ReadImageFromFieldToPicture(Field:TBlobField;Img:TPicture;ImageType:string;Compress:boolean);
procedure WriteImageToField(Field:TBlobField;Img:TImage;ImageType:string;Compress:boolean);
procedure ReadDataFromFile(Field:TBlobField;FileName:string;Compress:boolean);
procedure WriteDataToFile(Field:TBlobField;FileName:string;Compress:boolean);
procedure ConvertStreamToHex(FromStream,ToStream:TStream);

implementation

uses ZLib, Jpeg, SysUtils;

procedure CompressStream(aSource,aTarget:TStream);
var compStream:TCompressionStream;
begin
  compStream:=TCompressionStream.Create(clFastest,aTarget);
  try
    aSource.Position:=0;
    compStream.CopyFrom(aSource,aSource.Size);
    compStream.CompressionRate;
  finally
    compStream.Free;
  end;
end;

procedure DecompressStream(aSource,aTarget:TStream);
var decompStream:TDecompressionStream;
    nRead:integer;
    Buffer:array[0..1023] of Char;
    Count:integer;
begin
  decompStream:=TDecompressionStream.Create(aSource);
  try
    if aSource.Size > 0 then begin
      decompStream.Position:=0;
      Count:=1024;
      repeat

        try
          nRead:=decompStream.Read(Buffer,Count);
        except
          on E:Exception do begin
            raise Exception.Create('Decompression error when write ir to stream!');
          end;
        end;

        try
          aTarget.WriteBuffer(Buffer,nRead);
        except
          on E:Exception do begin
            raise Exception.Create('Decompression error when write ir to stream!');
          end;
        end;

      until nRead=0;
    end;
  finally
    decompStream.Free;
  end;
end;                    

procedure ReadDataFromField(Field:TBlobField;DataStream:TMemoryStream;DeCompress:boolean);
var compStream:TMemoryStream;
begin
  if Field.IsNull
  then DataStream.Clear
  else begin
    if DeCompress
    then compStream:=TMemoryStream.Create
    else compStream:=nil;
    try
      try
        if DeCompress
        then begin
          Field.SaveToStream(compStream);
          compStream.Position:=0;
          DecompressStream(compStream,DataStream);
        end
        else Field.SaveToStream(DataStream);
        DataStream.Position:=0;
      except
        on E:Exception do begin
          DataStream.Clear;
          if DeCompress
          then compStream.Clear;
        end;
      end;
    finally
      if DeCompress
      then compStream.Free;
    end;
  end;
end;

procedure WriteDataToField(Field:TBlobField;DataStream:TMemoryStream;Compress:boolean);
var compStream:TMemoryStream;
begin             
   if Compress
   then compStream:=TMemoryStream.Create;
   try
      if DataStream.Size=0
      then Field.Clear
      else begin
        DataStream.Position:=0;
        if Compress
        then begin
          CompressStream(DataStream,compStream);
          compStream.Position:=0;
          Field.LoadFromStream(compStream);
        end
        else begin
          DataStream.Position:=0;
          Field.LoadFromStream(DataStream);
        end;
        DataStream.Position:=0;
      end;
   finally
      if Compress
      then compStream.Free;
   end;
end;

procedure WriteImageToField(Field:TBlobField;Img:TImage;ImageType:string;Compress:boolean);
var mstream:TMemoryStream;
    jpeg:TJpegImage;
begin
  mstream:=TMemoryStream.Create;
  jpeg:=TJpegImage.Create;
  try
    if UpperCase(ImageType)='.JPG'
    then begin
      jpeg.Assign(Img.Picture.Graphic);
      jpeg.SaveToStream(mstream);
    end
    else
    if UpperCase(ImageType)='.ICO'
    then Img.Picture.Icon.SaveToStream(mstream)
    else Img.Picture.Bitmap.SaveToStream(mstream);
    WriteDataToField(Field,mstream,Compress);
  finally
    mstream.Free;
    jpeg.Free;
  end;
end;

procedure ReadImageFromField(Field:TBlobField;Img:TImage;ImageType:string;Compress:boolean);
var mstream:TMemoryStream;
    jpeg:TJpegImage;
begin
  mstream:=TMemoryStream.Create;
  jpeg:=TJpegImage.Create;
  try
    ReadDataFromField(Field,mstream,Compress);
    if mstream.Size>0
    then begin
      if UpperCase(ImageType)='.JPG'
      then begin
        jpeg:=TJpegImage.Create;
        jpeg.LoadFromStream(mstream);
        Img.Picture.Graphic:=jpeg;
      end
      else
      if UpperCase(ImageType)='.ICO'
      then Img.Picture.Icon.LoadFromStream(mstream)
      else Img.Picture.Bitmap.LoadFromStream(mstream);
    end
    else Img.Picture.Bitmap:=nil;
  finally
    mstream.Free;
    jpeg.Free;
  end;
end;

procedure ReadImageFromFieldToPicture(Field:TBlobField;Img:TPicture;ImageType:string;Compress:boolean);
var mstream:TMemoryStream;
    jpeg:TJpegImage;
begin
  mstream:=TMemoryStream.Create;
  jpeg:=TJpegImage.Create;
  try
    ReadDataFromField(Field,mstream,Compress);
    if mstream.Size>0
    then begin
      if UpperCase(ImageType)='.JPG'
      then begin
        jpeg:=TJpegImage.Create;
        jpeg.LoadFromStream(mstream);
        Img.Graphic:=jpeg;
      end
      else
      if UpperCase(ImageType)='.ICO'
      then Img.Icon.LoadFromStream(mstream)
      else Img.Bitmap.LoadFromStream(mstream);
    end
    else Img.Bitmap:=nil;
  finally
    mstream.Free;
    jpeg.Free;
  end;
end;

procedure WriteDataToFile(Field:TBlobField;FileName:string;Compress:boolean);
var mstream:TMemoryStream;
begin
  mstream:=TMemoryStream.Create;
  try
    ReadDataFromField(Field,mstream,Compress);
    mstream.SaveToFile(FileName);
  finally
    mstream.Free;
  end;
end;

procedure ReadDataFromFile(Field:TBlobField;FileName:string;Compress:boolean);
var mstream:TMemoryStream;
begin
  mstream:=TMemoryStream.Create;
  try
    mstream.LoadFromFile(filename);
    WriteDataToField(Field,mstream,Compress);
  finally
    mstream.Free;
  end;
end;

procedure ConvertStreamToHex(FromStream,ToStream:TStream);
var b:byte;
    s:string;
begin
  FromStream.Seek(0,soFromBeginning);
  while FromStream.Position<FromStream.Size do begin
    FromStream.Read(b,1);
    s:=UpperCase(IntToHex(b,2));
    ToStream.Write(s[1],Length(s));
  end;
end;

end.
